package foundations;
import java.util.*;

/**
  * Implements PositionableList with no duplicates and links pointing forwards and backwards
  * Student must implement removeBefore and removeAfter.
*/
public class DoublyLinkedListE extends DoublyLinkedDequeue implements PositionableList {

    // Fields are inherited from DoublyLinkedDequeue

   /** Insert obj after target object in the list */
	public void addAfter(Object obj, Object target) {
        if (this.contains(target) && !this.contains(obj)) {
            Node itemNode = getNode (target);
            // Backlink to itemNode and forward link to itemNode.next
            Node newNode = new Node (obj, itemNode.next, itemNode);
            if (itemNode.next != null)
                itemNode.next.before = newNode;
            else
                rear = newNode;
            itemNode.next = newNode;
            numberElements++;
        }
    }

	/** Insert obj before target object in the list */
	public void addBefore(Object obj, Object target) {
        if (this.contains(target) && !this.contains(obj)) {
            Node itemNode = getNode(target);
            Node beforeNode = itemNode.before;
            // Backlink to beforeNode forward link to itemNode
            Node newNode = new Node (obj, itemNode, beforeNode);
            if (front == itemNode)
                front = newNode;
            else
                beforeNode.next = newNode;
            itemNode.before = newNode;
            numberElements++;
        }
    }

	/** Delete object after target object in the list
	*   Throw NoSuchElementException if target not in the list.
	*   Throw NoSuchElementException if target is last in the list.
	*/
	public void removeAfter(Object target) {
        // Exercise
    }

	/** Delete object before target object in the list
	*   Throw NoSuchElementException if target not in the list.
	*   Throw NoSuchElementException if target is first in the list.
	*/
	public void removeBefore(Object target) {
        // Exercise
    }

	// Queries

	/** Return object after target object in the list
	*   Throw NoSuchElementException if target not in the list.
	*   Throw NoSuchElementException if target is last in the list.
	*/
    public Object elementAfter(Object target) {
        if (!this.contains(target) || getNode(target) == this.rear)
            throw new NoSuchElementException("removeAfter::obj does not exist or is last in list");
        else
            return getNode(target).next.item;
    }
        

	/** Return object before target object in the list
	*   Throw NoSuchElementException if target not in the list.
	*   Throw NoSuchElementException if target is first in the list.
	*/
	public Object elementBefore(Object target) {
        if (!this.contains(target) || getNode(target) == this.front)
            throw new NoSuchElementException("removeBefore::obj does not exist or is first in list");
        else
            return getNode(target).before.item;
    }

    // Internal methods

    /**
      * For internal use only.
      * This function, available only within this class
      * returns the node associated with value. If value is
      * not present in the list getNode returns null
    */
    protected Node getNode (Object value) {
        Node node = front;
        Node result = null;
        while (node != null) {
            if (node.item.equals(value)) {
                result = node;
                break;
            }
            node = node.next;
        }
        return result;
    }

    static public void main(String[] args) {
        DoublyLinkedListE myList = new DoublyLinkedListE();
        myList.addFront(new Integer(3));
        myList.addFront(new Integer(2));
        myList.addFront(new Integer(1));
        myList.addRear(new Integer(4));
        myList.addRear(new Integer(6));
        myList.addRear(new Integer(8));
        myList.addBefore(new Integer(6), new Integer(5));
        myList.addAfter(new Integer(6), new Integer(7));
        Iterator iter = myList.elements();
        while (iter.hasNext())
            System.out.println(iter.next());
        System.out.println("Element before 6 is " + myList.elementBefore(new Integer(6)));
        System.out.println("Elment after 6 is " + myList.elementAfter(new Integer(6)));
    }
} 
