import foundations.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
//import com.sun.java.swing.*;

public class ExprTreeUI extends JFrame {

    // Model
    
    ExpressionBinaryTree exprTree = new ExpressionBinaryTree();
    DrawTree dt;

	// Components
	
    JPanel actionPanel = new JPanel();
    JScrollPane scrollPane = new JScrollPane();
    JLabel labelMessages = new JLabel();
    JLabel labelAction = new JLabel();
    JTextArea textArea = new JTextArea();
    JButton buttonConstruct = new JButton();
    JLabel labelInfix = new JLabel();
    JLabel labelPostfix = new JLabel();
    JLabel labelPrefix = new JLabel();
    JTextField tfInfix = new JTextField();
    JTextField tfPostfix = new JTextField();
    JTextField tfPrefix = new JTextField();
    JLabel labelWarning = new JLabel();
    JScrollPane scrollDisplay = new JScrollPane();
    JPanel displayPanel = new JPanel();

    //Construct the frame

    public ExprTreeUI() {
        enableEvents(AWTEvent.WINDOW_EVENT_MASK);
        try  {
            jbInit();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }
    
	//Component initialization
    
    private void jbInit() throws Exception  {
        this.getContentPane().setLayout(null);
        this.setSize(new Dimension(448, 580));
        actionPanel.setBackground(new Color(255, 255, 200));
        actionPanel.setLayout(null);
        actionPanel.setBounds(new Rectangle(6, 28, 217, 241));
        scrollPane.setBackground(Color.white);
        scrollPane.setBounds(new Rectangle(229, 28, 206, 241));
        labelMessages.setText("Messages");
        labelMessages.setBounds(new Rectangle(301, 7, 62, 15));
        labelAction.setText("Actions");
        labelAction.setBounds(new Rectangle(91, 7, 46, 15));
        buttonConstruct.setText("Construct");
        buttonConstruct.setBounds(new Rectangle(61, 17, 95, 23));
        buttonConstruct.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                buttonConstruct_actionPerformed(e);
            }
        });
        labelInfix.setText("Infix Expression");
        labelInfix.setBounds(new Rectangle(12, 91, 111, 18));
        labelPostfix.setText("Postfix Expression");
        labelPostfix.setBounds(new Rectangle(12, 140, 126, 18));
        labelPrefix.setText("Prefix Expression");
        labelPrefix.setBounds(new Rectangle(12, 188, 126, 18));
        tfInfix.setBounds(new Rectangle(12, 113, 178, 21));
        tfInfix.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                tfInfix_actionPerformed(e);
            }
        });
        tfPostfix.setBounds(new Rectangle(12, 161, 178, 21));
        tfPostfix.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                tfPostfix_actionPerformed(e);
            }
        });
        tfPrefix.setEditable(false);
        tfPrefix.setBounds(new Rectangle(12, 208, 178, 21));
        labelWarning.setText("Expressions must be correct");
        labelWarning.setBounds(new Rectangle(12, 56, 174, 20));
        scrollDisplay.setBounds(new Rectangle(5, 278, 429, 271));
        displayPanel.setBackground(Color.white);
        tfPrefix.setBackground(Color.lightGray);
        this.setTitle("Expression Tree Laboratory");
                this.getContentPane().add(labelMessages, null);
        this.getContentPane().add(scrollPane, null);
        scrollPane.getViewport().add(textArea, null);
        this.getContentPane().add(labelAction, null);
        this.getContentPane().add(actionPanel, null);
        actionPanel.add(labelInfix, null);
        actionPanel.add(labelPrefix, null);
        actionPanel.add(labelPostfix, null);
        actionPanel.add(tfPrefix, null);
        actionPanel.add(tfPostfix, null);
        actionPanel.add(tfInfix, null);
                actionPanel.add(labelWarning, null);
        actionPanel.add(buttonConstruct, null);
        this.getContentPane().add(scrollDisplay, null);
        scrollDisplay.getViewport().add(displayPanel, null);
        enableWidgets(false);
        dt = new DrawTree(exprTree.root(), exprTree.size(), displayPanel, Color.blue, 4);
    }

	//Overriden so we can exit on System Close
    
    protected void processWindowEvent(WindowEvent e) {
        super.processWindowEvent(e);
        if (e.getID() == WindowEvent.WINDOW_CLOSING) {
            System.exit(0);
        }
    }

    void enableWidgets(boolean b){
        labelInfix.setVisible(b);
        labelPostfix.setVisible(b);
        labelPrefix.setVisible(b);
        labelWarning.setVisible(b);
        tfInfix.setVisible(b);
        tfPostfix.setVisible(b);
        tfPrefix.setVisible(b);
        displayPanel.setVisible(b);
    }

    void updateExpressions(){
       	tfInfix.setText(exprTree.traverseInorder());
       	tfPostfix.setText(exprTree.traversePostorder());
       	tfPrefix.setText(exprTree.traversePreorder());
        dt.update(exprTree.root(), exprTree.size());
    }

    void clearDisplay(){
    	tfInfix.setText("");
    	tfPostfix.setText("");
    	tfPrefix.setText("");
    	dt.clearPanel();
    }

    void buttonConstruct_actionPerformed(ActionEvent e) {
        exprTree = new ExpressionBinaryTree("");
        enableWidgets(true);
        buttonConstruct.setEnabled(false);
        tfInfix.requestFocus();
    }

    void tfInfix_actionPerformed(ActionEvent e) {
        try {
        	String infix = tfInfix.getText().trim();
        	exprTree.setInfixString(infix);
        	updateExpressions();
        	textArea.append("From infix string\n");
        } catch (Exception ex) { 
        	textArea.append("Error in infix string\n");
        	clearDisplay();
        }
    }

    void tfPostfix_actionPerformed(ActionEvent e) {
        try {
        	String postfix = tfPostfix.getText().trim();
        	exprTree.setPostfixString(postfix);
        	updateExpressions();
        	textArea.append("From postfix string\n");
        } catch (Exception ex) { 
        	textArea.append("Error in postfix string\n");
        	clearDisplay();
        }
    }
}

