/** Simulating the performance of coalesced chaining.
*/
import java.text.*;
import java.util.*;

public class CoalescedChainingApp {

    // Fields
    final int SIZE = 1000;
    int probes;
    boolean [] words = new boolean [SIZE];
    int [] link = new int [SIZE];
    Random rnd = new Random();
    int count;
    int cri; // collision resolution index

    public void generateTable (double loadFactor) {
        // Empty the table
        for (int i = 0; i < SIZE; i++)
            words[i] = false;
        // Reset the link array
        for (int i = 0; i < SIZE; i++)
            link[i] = -1;
        cri = SIZE - 1;
        count = (int) (SIZE * loadFactor);
        probes = 0;
        for (int i = 0; i < count; i++) {
            int hashIndex = rnd.nextInt(SIZE); 
	   		if (words[hashIndex] == false) {
                words[hashIndex] = true;
                probes++;
            }
            else { // collision resolution required
                // Scan to end of collision chain
                int index = hashIndex;
                while (link[index] != -1) {
                    probes++;
                    index = link[index];
                }
                // Find cri
                while (words[cri] == true)
                    cri--;
                words[cri] = true;
                link[index] = cri;
                probes++;
            }
        }
    }

    static public void main(String[] args) {
        DecimalFormat df = new DecimalFormat("#.00");
        CoalescedChainingApp app = new CoalescedChainingApp();
        System.out.println ("LF\t\t\t\tAverage Number of Probes");
        for (double lf = 0.1; lf <= 0.9; lf += 0.1) {
            app.generateTable(lf);
            System.out.println(df.format(lf) + "\t\t\t\t" + 
                      df.format((double) app.probes / app.count));
        }
        for (double lf = 0.91; lf < 1.0; lf += 0.01) {
            app.generateTable(lf);
            System.out.println(df.format(lf) + "\t\t\t\t" + 
                      df.format((double) app.probes / app.count));
        }
    }
}

