/** An application that simulates the performance of linear chaining.
*/
import java.text.*;
import java.util.*;

public class LinearChainingApp {

    // Fields
    int probes;
    boolean [] words = new boolean [1000];  // Holds virtual words
    Random rnd = new Random();
    int count;

    public void generateTable (double loadFactor) {
        // Empty the table
        for (int i = 0; i < 1000; i++)
            words[i] = false;
        count = (int) (1000 * loadFactor);
        probes = 0;
        for (int i = 0; i < count; i++) {
            int hashIndex = rnd.nextInt(1000); 
            while (words[hashIndex] == true) {
                hashIndex++;
                probes++;
                if (hashIndex == 1000)
                    hashIndex = 0;
            }
            words[hashIndex] = true;
            probes++;
        }
    }

    static public void main (String[] args) {
        DecimalFormat df = new DecimalFormat("#.00");
        LinearChainingApp app = new LinearChainingApp();
        System.out.println (
				"Load Factor\t\t\tAverage Number of Probes");
        for (double lf = 0.1; lf <= 0.9; lf += 0.1) {
            app.generateTable(lf);
            System.out.println(df.format(lf) + "\t\t\t\t" + 
                          df.format((double) app.probes / app.count));
        }

        for (double lf = 0.91; lf < 1.0; lf += 0.01) {
            app.generateTable(lf);
            System.out.println(df.format(lf) + "\t\t\t\t" + 
                          df.format((double) app.probes / app.count));
        }
    }
}

