import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.text.*;

public class SortLabGUI extends Frame {
    Choice sortMethod = new Choice();
    Label label1 = new Label();
    TextField arraySize = new TextField();
    Label label2 = new Label();
    TextField timeDuration = new TextField();
    Label label3 = new Label();
    Checkbox noTiming = new Checkbox();
    Button sort = new Button();
    TextField error = new TextField();
    Label label4 = new Label();

    // Internal field
    private double [] data = new double [1000001]; // Used to hold data for sorting
    private Random r = new Random();
    private TimeInterval t = new TimeInterval();
    private Image bufferedImage;
    public boolean display;

    public SortLabGUI() {
        enableEvents(AWTEvent.WINDOW_EVENT_MASK);
        try  {
            jbInit();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }
//Component initialization
    
    private void jbInit() throws Exception  {
        this.setLayout(null);
        this.setBackground(Color.lightGray);
        this.setSize(new Dimension(800, 475));
        this.setTitle("Sort Laboratory");
        this.addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowOpened(WindowEvent e) {
                this_windowOpened(e);
            }
        });

        sortMethod.setBounds(new Rectangle(25, 60, 120, 25));
        
        sortMethod.addItemListener(new java.awt.event.ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                sortMethod_itemStateChanged(e);
            }
        });

        label1.setForeground(Color.red);
        label1.setBounds(new Rectangle(42, 38, 87, 23));
        label1.setText("Sorting Method");
        arraySize.setBounds(new Rectangle(25, 146, 120, 23));
        label2.setForeground(Color.red);
        label2.setBounds(new Rectangle(55, 124, 60, 23));
        label2.setText("Array Size");
        timeDuration.setBounds(new Rectangle(25, 315, 120, 23));
        timeDuration.setEditable (false);
        label3.setForeground(Color.red);
        label3.setBounds(new Rectangle(44, 291, 83, 23));
        label3.setText("Time Duration");
        noTiming.setBounds(new Rectangle(25, 231, 172, 23));
        noTiming.setLabel("Display array and no timing");
        sort.setForeground(Color.red);
        sort.setBounds(new Rectangle(325, 36, 151, 23));
        sort.setLabel("Perform Sorting");
        error.setBounds(new Rectangle(25, 393, 120, 23));
        error.setEditable (false);
        label4.setForeground(Color.red);
        label4.setBounds(new Rectangle(69, 366, 33, 23));
        label4.setText("Error");
        sort.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(ActionEvent e) {
                sort_actionPerformed(e);
            }
        });
        this.add(sortMethod, null);
        this.add(label1, null);
        this.add(arraySize, null);
        this.add(label2, null);
        this.add(timeDuration, null);
        this.add(label3, null);
        this.add(noTiming, null);
        this.add(sort, null);
        this.add(error, null);
        this.add(label4, null);
    }

    //Overriden so we can exit on System Close
    protected void processWindowEvent(WindowEvent e) {
        super.processWindowEvent(e);
        if (e.getID() == WindowEvent.WINDOW_CLOSING) {
            System.exit(0);
        }
    }

    void this_windowOpened(WindowEvent e) {
        sortMethod.add ("Selection sort");
        sortMethod.add ("Bubble sort");
        sortMethod.add ("Gap sort");
        sortMethod.add ("Quick sort");
        noTiming.setState (true);
        SortMethods.lab = this;
    }

    void displayData (int size) {
        int xInterval = size / 500;
        int x;

        if (bufferedImage == null)
             bufferedImage = createImage (800, 475);
        Graphics bg = bufferedImage.getGraphics();
        Graphics g = this.getGraphics();
        bg.setColor (Color.lightGray);
        bg.fillRect (0, 0, 800, 475);
        bg.setColor (Color.black);
        for (int index = 1; index <= size; index++)
            if (index % xInterval == 0) {
                x = index / xInterval;
                bg.drawLine (225 + x, 450, 225 + x,
                             450 - (int) (data [index] * 350.0));
            }
        bg.dispose();
        g.drawImage (bufferedImage, 0, 0, null);
    }

    void sort_actionPerformed(ActionEvent e) {
        DecimalFormat df = new DecimalFormat ("0.###");
        if (arraySize.getText().trim().length() == 0)
            error.setText("Array size blank");
        else {
            error.setText ("");
            try {
                int size = (new Integer (arraySize.getText().trim())).intValue();
                if (size <= 1000000) {
                    for (int index = 1; index <= size; index++)
                        data [index] = r.nextDouble();
                    if (noTiming.getState() == false) {
                        timeDuration.setText ("Sorting ...");
                        displayData (size);
                        t.startTiming();
                        display = false;
                        switch (sortMethod.getSelectedIndex()) {
                            case 0: SortMethods.selectionSort (data, size);
                                    break;
                            case 1: SortMethods.bubbleSort (data, size);
                                    break;
                            case 2: SortMethods.gapSort (data, size);
                                    break;
                            case 3: SortMethods.quickSort (data, 1, size);
                                    break;
                            default: break;
                        }
                        t.endTiming();
                        displayData (size);
                        timeDuration.setText (df.format (t.getElapsedTime()) +
                                              " seconds");
                    }
                    else {
                        display = true;
                        displayData (size);
                        timeDuration.setText ("");
                        error.setText ("");
                        switch (sortMethod.getSelectedIndex()) {
                            case 0: SortMethods.selectionSort (data, size);
                                    break;
                            case 1: SortMethods.bubbleSort (data, size);
                                    break;
                            case 2: SortMethods.gapSort (data, size);
                                    break;
                            case 3: SortMethods.size = size;
                                    SortMethods.quickSort (data, 1, size);
                                    break;
                            default: break;
                        }
                        displayData (size);
                    }
                }
                else
                    error.setText ("Size too large");
            }
            catch (Exception ex) {
                error.setText ("Illegal array size");
                timeDuration.setText ("");
            }
        }

    }

    void sortMethod_itemStateChanged(ItemEvent e) {
        if (sortMethod.getSelectedIndex() <= 1)
            arraySize.setText ("5000");
        else
            arraySize.setText ("1000000");
    }
}
 