/**
  * Holds several important static sorting methods
  * Assume that data is stored in indices 1 to size for all methods
*/

import java.util.*;

public class SortMethods {

    // Fields
    public static SortLabGUI lab;
    public static int size;

    public static void selectionSort (double [] data, int size) {
        int maxIndex, index1, index2;
        double max, temp;

        for (index1 = size; index1 >= 2; index1--) {
            max = data [1];
            maxIndex = 1;
            for (index2 = 2; index2 <= index1; index2++)
                if (data [index2] > max) {
                    maxIndex = index2;
                    max = data [index2];
                }
            temp = data [index1];
            data [index1] = data [maxIndex];
            data [maxIndex] = temp;
            if (lab.display && index1 % 500 == 0)
                lab.displayData(size);
        }
    }

    // Static methods
    public static void bubbleSort (double [] data, int size) {
        int index1, index2;
        double temp;
        boolean exchanged;

        for (index1 = size; index1 >= 2; index1--) {
            exchanged = false;
            for (index2 = 1; index2 <= index1 - 1; index2++) {
                if (data [index2] > data [index2 + 1]) {
                    temp = data [index2];
                    data [index2] = data [index2 + 1];
                    data [index2 + 1] = temp;
                    exchanged = true;
                }
            }
            if (lab.display && index1 % 500 == 0)
                lab.displayData (size);
            if (!exchanged)
                break;
        }
    }

    public static void quickSort (double [] data, int low, int high) {
        int partitionIndex;
        if (high - low > 0) {
            partitionIndex = partition (data, low, high);
            if (lab.display && high - low > 25000)
                lab.displayData (size);
            quickSort (data, low, partitionIndex - 1);
            quickSort (data, partitionIndex + 1, high);
        }
    }

    private static int partition (double [] data, int low, int high) {
        int k, j;
        double temp, p;

        p = data [low]; // Partition element
        k = low;
        j = high + 1;
        do {
            k++;
        } while (data [k] <= p && k < high);
        do {
            j--;
        } while (data [j] > p);
        while (k < j) {
            temp = data [k];
            data [k] = data [j];
            data [j] = temp;
            do {
                k++;
            } while (data [k] <= p);
            do {
                j--;
            } while (data [j] > p);
        }
        if (low != j) {
            temp = data [low];
            data [low] = data [j];
            data [j] = temp;
        }
        return j;
    }

    public static void gapSort (double [] data, int size) {
        int index;
        int count = 0;
        int gap, top;
        double temp;
        boolean exchanged;
        double SF = 1.3;

        gap = size;
        do {
            count++;
            exchanged = false;
            gap = (int) (gap / SF);
            if (gap == 0)
                gap = 1;
            for (index = 1; index <= size - gap; index++) {
                if (data [index] > data [index + gap]) {
                    temp = data [index];
                    data [index] = data [index + gap];
                    data [index + gap] = temp;
                    exchanged = true;
                }
            }
            if (lab.display && count % 5 == 0)
                lab.displayData (size);
        } while (exchanged || gap > 1);
    }

    public static boolean binarySearch (double [] data, int first, int last,
                                        double value) {
        int middle = (first + last) / 2;
        if (data [middle] == value)
            return true;
        else if (value < data [middle]) {
            if (first <= middle - 1)
                return binarySearch (data, first, middle - 1, value);
            else
                return false;
        }
        else if (value > data [middle]) {
            if (middle + 1 <= last)
                return binarySearch (data, middle + 1, last, value);
            else
                return false;
        }
        return false; // This line is never reached but needed for compiler
    }
}