// File CounterBeanGUI.java - Delegation MVC Example - 3/25/98
import java.awt.event.*;
import java.awt.*;
import java.beans.*;
import javax.swing.*;

public class CounterBeanUI extends JFrame implements PropertyChangeListener, ActionListener {

    protected CounterBean counter = new CounterBean(); 
    protected Button resetButton = new Button("Reset"); 
    protected Button incrementButton = new Button("Increment"); 
    protected Label labelCount = new Label("Current value of count:   ", 1); 

    public CounterBeanUI(String title){
        super(title);
        init();
    }
    
    public void init () {
        setSize(220, 100);
        getContentPane().setBackground(new Color(255, 255, 200));
        getContentPane().setLayout(new BorderLayout());
        getContentPane().add(labelCount, "Center");
        
        Panel buttonPanel = new Panel();
        buttonPanel.add(incrementButton);
        buttonPanel.add(resetButton);
        getContentPane().add(buttonPanel, "South");
    
        counter.addPropertyChangeListener(this);
        incrementButton.addActionListener(this);
        resetButton.addActionListener(this);
        center();
        setVisible(true);
    }

    protected void processWindowEvent (WindowEvent e) {
        super.processWindowEvent(e);
        if (e.getID() == WindowEvent.WINDOW_CLOSING) {
            System.exit(0);
        }
    }

    public void center () {
    	//Center the window
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension frameSize = this.getSize();
        if (frameSize.height > screenSize.height)
            frameSize.height = screenSize.height;
        if (frameSize.width > screenSize.width)
            frameSize.width = screenSize.width;
        this.setLocation((screenSize.width - frameSize.width) / 2, 
        					(screenSize.height - frameSize.height) / 2);
    }

    /**
     * Event Handling methods
     */
    public void propertyChange (PropertyChangeEvent evt) {
        if ((evt.getSource() == counter) && (evt.getPropertyName().equals("count"))) {
            labelCount.setText("Current value of count:  " + String.valueOf(counter.getCount()));
        }
    }
    
    public void actionPerformed (ActionEvent evt) {
        if (evt.getSource() == resetButton)
            counter.reset();	
        else if (evt.getSource() == incrementButton)
            counter.increment();
    }
	
    public static void main (String args[]) {
        new CounterBeanUI("MVC-Bean Counter");
    }
}



