import java.awt.event.*;
import java.awt.*;
import java.util.*;
import javax.swing.*;

public class CounterUI extends JFrame implements Observer, ActionListener {
    protected CounterModel counter = new CounterModel(); 
    protected Button resetButton = new Button("Reset"); 
    protected Button incrementButton = new Button("Increment"); 
    protected Label labelCount = new Label("Current value of count:   ", 1); 

    /**
     *  Create and initialize an instance of CounterUI
     */
    public CounterUI(String arg1) {
        super (arg1);
        init();
    }
	
    public void init () {
        setSize(220, 100);
        getContentPane().setBackground(new Color(255, 255, 200));
        getContentPane().setLayout(new BorderLayout());
        getContentPane().add(labelCount, "Center");
        
        Panel buttonPanel = new Panel();
        buttonPanel.add(incrementButton);
        buttonPanel.add(resetButton);
        getContentPane().add(buttonPanel, "South");
        
        counter.addObserver(this);
        resetButton.addActionListener(this);
        incrementButton.addActionListener(this);
        center();
        setVisible(true);
    }	


    protected void processWindowEvent (WindowEvent e) {
        super.processWindowEvent(e);
        if (e.getID() == WindowEvent.WINDOW_CLOSING) {
            System.exit(0);
        }
    }

    public void center () {
    	//Center the window
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        Dimension frameSize = this.getSize();
        if (frameSize.height > screenSize.height)
            frameSize.height = screenSize.height;
        if (frameSize.width > screenSize.width)
            frameSize.width = screenSize.width;
        this.setLocation((screenSize.width - frameSize.width) / 2, 
        					(screenSize.height - frameSize.height) / 2);
    }
		
    /**
     * Add Methods for View-Controller functions
     */
    public void update (Observable o, Object arg) {
        if ( o instanceof CounterModel) 
            labelCount.setText("Current value of count:  " + String.valueOf(counter.getCount()));
    }
    
    /**
     * Controller actions - event handling
     */
    public void actionPerformed (ActionEvent evt) {
        if (evt.getSource() == resetButton)
            counter.reset();	
        else if (evt.getSource() == incrementButton)
            counter.increment();
    }
		
    /**
     *  Invoke this as a stand alone application.
     */
    public static void main (String args[]) {
        new CounterUI("MVC-Counter");

    }
}
